@@ -21,7 +21,7 @@ def run(options):
     init = {}
     for db_name in options.database_names:
         init[db_name] = False
-        with Transaction().start(db_name, 0):
+        with Transaction().start(db_name, 0, _nocache=True):
             database = Database(db_name)
             database.connect()
             if options.update:
