@@ -1,8 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from collections import namedtuple
+
 DatabaseIntegrityError = None
 DatabaseOperationalError = None
 
+SQLType = namedtuple('SQLType', 'base type')
+
 
 class DatabaseInterface(object):
     '''
@@ -151,3 +155,19 @@ class DatabaseInterface(object):
     def has_multirow_insert(self):
         'Return True if database supports multirow insert'
         return False
+
+    def has_select_for(self):
+        "Return if database supports FOR UPDATE/SHARE clause in SELECT."
+        return False
+
+    def has_window_functions(self):
+        "Return if database supports window functions."
+        return False
+
+    def sql_type(self, type_):
+        'Return the SQLType tuple corresponding to the SQL type'
+        pass
+
+    def sql_format(self, type_, value):
+        'Return value correctly casted into type_'
+        pass
