@@ -14,6 +14,9 @@ class UserError(TrytonException):
         self.description = description
         self.code = 1
 
+    def __str__(self):
+        return '%s - %s' % (self.message, self.description)
+
 
 class UserWarning(TrytonException):
 
@@ -25,6 +28,9 @@ class UserWarning(TrytonException):
         self.description = description
         self.code = 2
 
+    def __str__(self):
+        return '%s - %s' % (self.message, self.description)
+
 
 class LoginException(TrytonException):
     """Request the named parameter for the login process.
@@ -47,3 +53,10 @@ class ConcurrencyException(TrytonException):
             message)
         self.message = message
         self.code = 4
+
+    def __str__(self):
+        return self.message
+
+
+class RateLimitException(TrytonException):
+    """User has sent too many requests in a given amount of time."""
