@@ -356,7 +356,6 @@ class ModelField(ModelSQL, ModelView):
 class ModelAccess(ModelSQL, ModelView):
     "Model access"
     __name__ = 'ir.model.access'
-    _rec_name = 'model'
     model = fields.Many2One('ir.model', 'Model', required=True,
             ondelete="CASCADE")
     group = fields.Many2One('res.group', 'Group',
@@ -412,6 +411,13 @@ class ModelAccess(ModelSQL, ModelView):
     def default_perm_delete():
         return False
 
+    def get_rec_name(self, name):
+        return self.model.rec_name
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        return [('model',) + tuple(clause[1:])]
+
     @classmethod
     def get_access(cls, models):
         'Return access for models'
@@ -540,7 +546,6 @@ class ModelAccess(ModelSQL, ModelView):
 class ModelFieldAccess(ModelSQL, ModelView):
     "Model Field Access"
     __name__ = 'ir.model.field.access'
-    _rec_name = 'field'
     field = fields.Many2One('ir.model.field', 'Field', required=True,
             ondelete='CASCADE')
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE')
@@ -590,6 +595,13 @@ class ModelFieldAccess(ModelSQL, ModelView):
     def default_perm_delete():
         return True
 
+    def get_rec_name(self, name):
+        return self.field.rec_name
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        return [('field',) + tuple(clause[1:])]
+
     @classmethod
     def get_access(cls, models):
         'Return fields access for models'
