@@ -10,7 +10,7 @@ from lxml import etree
 from itertools import izip
 from io import BytesIO
 
-from sql import Column, Null
+from sql import Column, Null, Literal
 from sql.functions import Substring, Position
 from sql.conditionals import Case
 from sql.operators import Or, And
@@ -36,6 +36,7 @@ __all__ = ['Translation',
     'TranslationCleanStart', 'TranslationCleanSucceed', 'TranslationClean',
     'TranslationUpdateStart', 'TranslationUpdate',
     'TranslationExportStart', 'TranslationExportResult', 'TranslationExport',
+    'TranslationReport',
     ]
 
 TRANSLATION_TYPE = [
@@ -104,10 +105,7 @@ class Translation(ModelSQL, ModelView):
         # Migration from 1.8: new field src_md5
         src_md5_exist = table.column_exist('src_md5')
         if not src_md5_exist:
-            table.add_raw_column('src_md5',
-                cls.src_md5.sql_type(),
-                cls.src_md5.sql_format, None,
-                cls.src_md5.size, string=cls.src_md5.string)
+            table.add_column('src_md5', cls.src_md5._sql_type)
         table.drop_constraint('translation_uniq')
         table.index_action(['lang', 'type', 'name', 'src'], 'remove')
 
@@ -433,7 +431,7 @@ class Translation(ModelSQL, ModelView):
         if not Transaction().context.get(
                 'fuzzy_translation', False):
             for obj_id in ids:
-                trans = cls._translation_cache.get((lang, ttype, name, obj_id),
+                trans = cls._translation_cache.get((name, ttype, lang, obj_id),
                     -1)
                 if trans != -1:
                     translations[obj_id] = trans
@@ -474,7 +472,7 @@ class Translation(ModelSQL, ModelView):
             value = translations.setdefault(res_id)
             # Don't store fuzzy translation in cache
             if not Transaction().context.get('fuzzy_translation', False):
-                cls._translation_cache.set((lang, ttype, name, res_id), value)
+                cls._translation_cache.set((name, ttype, lang, res_id), value)
         return translations
 
     @classmethod
@@ -514,16 +512,16 @@ class Translation(ModelSQL, ModelView):
                 else:
                     return record.model + ',' + field_name
 
-            translations = {}
-            for translation in cls.search([
-                        ('lang', '=', lang),
-                        ('type', '=', ttype),
-                        ('name', 'in', [get_name(r) for r in records]),
-                        ]):
-                translations[translation.name] = translation
-
-            to_save = []
             with Transaction().set_context(_check_access=False):
+                translations = {}
+                for translation in cls.search([
+                            ('lang', '=', lang),
+                            ('type', '=', ttype),
+                            ('name', 'in', [get_name(r) for r in records]),
+                            ]):
+                    translations[translation.name] = translation
+
+                to_save = []
                 for record, value in izip(records, values):
                     translation = translations.get(get_name(record))
                     if not translation:
@@ -543,28 +541,28 @@ class Translation(ModelSQL, ModelView):
             records = Model.browse(ids)
 
         translations = {}
-        for translation in cls.search([
-                    ('lang', '=', lang),
-                    ('type', '=', ttype),
-                    ('name', '=', name),
-                    ('res_id', 'in', ids),
-                    ]):
-            translations[translation.res_id] = translation
-
-        other_translations = {}
-        if (lang == Config.get_language()
-                and Transaction().context.get('fuzzy_translation', True)):
+        with Transaction().set_context(_check_access=False):
             for translation in cls.search([
-                        ('lang', '!=', lang),
+                        ('lang', '=', lang),
                         ('type', '=', ttype),
                         ('name', '=', name),
                         ('res_id', 'in', ids),
                         ]):
-                other_translations.setdefault(translation.res_id, []
-                    ).append(translation)
+                translations[translation.res_id] = translation
 
-        to_save = []
-        with Transaction().set_context(_check_access=False):
+            other_translations = {}
+            if (lang == Config.get_language()
+                    and Transaction().context.get('fuzzy_translation', True)):
+                for translation in cls.search([
+                            ('lang', '!=', lang),
+                            ('type', '=', ttype),
+                            ('name', '=', name),
+                            ('res_id', 'in', ids),
+                            ]):
+                    other_translations.setdefault(translation.res_id, []
+                        ).append(translation)
+
+            to_save = []
             for record, value in izip(records, values):
                 translation = translations.get(record.id)
                 if not translation:
@@ -636,7 +634,7 @@ class Translation(ModelSQL, ModelView):
             lang = unicode(lang)
             if source is not None:
                 source = unicode(source)
-            trans = cls._translation_cache.get((lang, ttype, name, source), -1)
+            trans = cls._translation_cache.get((name, ttype, lang, source), -1)
             if trans != -1:
                 res[(name, ttype, lang, source)] = trans
             else:
@@ -1059,78 +1057,85 @@ class TranslationSet(Wizard):
         pool = Pool()
         Report = pool.get('ir.action.report')
         Translation = pool.get('ir.translation')
+        context = Transaction().context
 
-        with Transaction().set_context(active_test=False):
-            reports = Report.search([])
-
-        if not reports:
+        if context.get('active_model') == Report.__name__:
+            reports = Report.browse(context.get('active_ids', []))
+        elif context.get('active_model', 'ir.ui.menu') == 'ir.ui.menu':
+            with Transaction().set_context(active_test=False):
+                reports = Report.search([])
+        else:
             return
 
         cursor = Transaction().connection.cursor()
         translation = Translation.__table__()
         for report in reports:
-            cursor.execute(*translation.select(
-                    translation.id, translation.name, translation.src,
-                    where=(translation.lang == 'en')
-                    & (translation.type == 'report')
-                    & (translation.name == report.report_name)
-                    & (translation.module == report.module or '')))
-            trans_reports = {t['src']: t for t in cursor_dict(cursor)}
-
             content = None
             if report.report:
                 with file_open(report.report.replace('/', os.sep),
                         mode='rb') as fp:
                     content = fp.read()
-            strings = []
-            for content in [report.report_content_custom, content]:
+            for content, module in [
+                    (report.report_content_custom, None),
+                    (content, report.module)]:
                 if not content:
                     continue
-                func_name = 'extract_report_%s' % report.template_extension
-                strings.extend(getattr(self, func_name)(content))
 
-            for string in {}.fromkeys(strings).keys():
-                src_md5 = Translation.get_src_md5(string)
-                done = False
-                if string in trans_reports:
-                    del trans_reports[string]
-                    continue
-                for string_trans in trans_reports:
-                    if string_trans in strings:
-                        continue
-                    seqmatch = SequenceMatcher(lambda x: x == ' ',
-                            string, string_trans)
-                    if seqmatch.ratio() == 1.0:
-                        del trans_reports[report.report_name][string_trans]
-                        done = True
-                        break
-                    if seqmatch.ratio() > 0.6:
-                        cursor.execute(*translation.update(
-                                [translation.src, translation.fuzzy,
-                                    translation.src_md5],
-                                [string, True, src_md5],
-                                where=(translation.name == report.report_name)
-                                & (translation.type == 'report')
-                                & (translation.src == string_trans)
-                                & (translation.module == report.module)))
-                        del trans_reports[string_trans]
-                        done = True
-                        break
-                if not done:
-                    cursor.execute(*translation.insert(
-                            [translation.name, translation.lang,
-                                translation.type, translation.src,
-                                translation.value, translation.module,
-                                translation.fuzzy, translation.src_md5,
-                                translation.res_id],
-                            [[report.report_name, 'en', 'report', string,
-                                    '', report.module, False, src_md5, -1]]))
-            if strings:
-                cursor.execute(*translation.delete(
-                        where=(translation.name == report.report_name)
+                cursor.execute(*translation.select(
+                        translation.id, translation.name, translation.src,
+                        where=(translation.lang == 'en')
                         & (translation.type == 'report')
-                        & (translation.module == report.module)
-                        & ~translation.src.in_(strings)))
+                        & (translation.name == report.report_name)
+                        & (translation.module == module)))
+                trans_reports = {t['src']: t for t in cursor_dict(cursor)}
+
+                strings = set()
+                func_name = 'extract_report_%s' % report.template_extension
+                strings.update(getattr(self, func_name)(content))
+
+                for string in strings:
+                    src_md5 = Translation.get_src_md5(string)
+                    done = False
+                    if string in trans_reports:
+                        del trans_reports[string]
+                        continue
+                    for string_trans in trans_reports:
+                        if string_trans in strings:
+                            continue
+                        seqmatch = SequenceMatcher(lambda x: x == ' ',
+                                string, string_trans)
+                        if seqmatch.ratio() == 1.0:
+                            del trans_reports[report.report_name][string_trans]
+                            done = True
+                            break
+                        if seqmatch.ratio() > 0.6:
+                            cursor.execute(*translation.update(
+                                    [translation.src, translation.fuzzy,
+                                        translation.src_md5],
+                                    [string, True, src_md5],
+                                    where=(
+                                        translation.name == report.report_name)
+                                    & (translation.type == 'report')
+                                    & (translation.src == string_trans)
+                                    & (translation.module == module)))
+                            del trans_reports[string_trans]
+                            done = True
+                            break
+                    if not done:
+                        cursor.execute(*translation.insert(
+                                [translation.name, translation.lang,
+                                    translation.type, translation.src,
+                                    translation.value, translation.module,
+                                    translation.fuzzy, translation.src_md5,
+                                    translation.res_id],
+                                [[report.report_name, 'en', 'report', string,
+                                        '', module, False, src_md5, -1]]))
+                if strings:
+                    cursor.execute(*translation.delete(
+                            where=(translation.name == report.report_name)
+                            & (translation.type == 'report')
+                            & (translation.module == module)
+                            & ~translation.src.in_(list(strings))))
 
     def _translate_view(self, element):
         strings = []
@@ -1147,12 +1152,16 @@ class TranslationSet(Wizard):
         pool = Pool()
         View = pool.get('ir.ui.view')
         Translation = pool.get('ir.translation')
+        context = Transaction().context
 
-        with Transaction().set_context(active_test=False):
-            views = View.search([])
-
-        if not views:
+        if context.get('active_model') == View.__name__:
+            views = View.browse(context.get('active_ids', []))
+        elif context.get('active_model', 'ir.ui.menu') == 'ir.ui.menu':
+            with Transaction().set_context(active_test=False):
+                views = View.search([])
+        else:
             return
+
         cursor = Transaction().connection.cursor()
         translation = Translation.__table__()
         for view in views:
@@ -1497,20 +1506,36 @@ class TranslationUpdate(Wizard):
     def do_update(self, action):
         pool = Pool()
         Translation = pool.get('ir.translation')
+        Report = pool.get('ir.action.report')
+        View = pool.get('ir.ui.view')
+        context = Transaction().context
         cursor = Transaction().connection.cursor()
         cursor_update = Transaction().connection.cursor()
         translation = Translation.__table__()
         lang = self.start.language.code
         parent_lang = get_parent(lang)
 
+        if context.get('active_model') == Report.__name__:
+            reports = Report.browse(context.get('active_ids', []))
+            source_clause = ((translation.type == 'report')
+                & translation.name.in_([r.report_name for r in reports]))
+        elif context.get('active_model') == View.__name__:
+            views = View.browse(context.get('active_ids', []))
+            source_clause = ((translation.type == 'view')
+                & translation.name.in_([v.model for v in views]))
+        else:
+            source_clause = Literal(True)
+
         columns = [translation.name.as_('name'),
             translation.res_id.as_('res_id'), translation.type.as_('type'),
             translation.src.as_('src'), translation.module.as_('module')]
         cursor.execute(*(translation.select(*columns,
                     where=(translation.lang == 'en')
+                    & source_clause
                     & translation.type.in_(self._source_types))
                 - translation.select(*columns,
                     where=(translation.lang == lang)
+                    & source_clause
                     & translation.type.in_(self._source_types))))
         to_create = []
         for row in cursor_dict(cursor):
@@ -1529,9 +1554,11 @@ class TranslationUpdate(Wizard):
             columns.append(translation.value)
             cursor.execute(*(translation.select(*columns,
                         where=(translation.lang == parent_lang)
+                        & source_clause
                         & translation.type.in_(self._source_types))
                     & translation.select(*columns,
                         where=(translation.lang == lang)
+                        & source_clause
                         & translation.type.in_(self._source_types))))
             for row in cursor_dict(cursor):
                 cursor_update.execute(*translation.update(
@@ -1544,6 +1571,9 @@ class TranslationUpdate(Wizard):
                         & (translation.module == row['module'])
                         & (translation.lang == lang)))
 
+        if context.get('active_model') in {Report.__name__, View.__name__}:
+            return
+
         columns = [translation.name.as_('name'),
             translation.res_id.as_('res_id'), translation.type.as_('type'),
             translation.module.as_('module')]
@@ -1698,3 +1728,28 @@ class TranslationExport(Wizard):
         return {
             'file': cast(file_) if file_ else None,
             }
+
+
+class TranslationReport(Wizard):
+    "Open translations of report"
+    __name__ = 'ir.translation.report'
+    start_state = 'open_'
+    open_ = StateAction('ir.act_translation_report')
+
+    def do_open_(self, action):
+        pool = Pool()
+        Report = pool.get('ir.action.report')
+        context = Transaction().context
+        assert context['active_model'] == Report.__name__
+        reports = Report.browse(context['active_ids'])
+        action['pyson_domain'] = PYSONEncoder().encode([
+                ('type', '=', 'report'),
+                ('name', 'in', [r.report_name for r in reports]),
+                ])
+        # Behaves like a relate to have name suffix
+        action['keyword'] = 'form_relate'
+        return action, {
+            'model': context['active_model'],
+            'ids': context['active_ids'],
+            'id': context['active_id'],
+            }
